# -*- coding: utf-8 -*-


__all__ = ["poly_to_valid",
           ]


def poly_to_valid(poly):
    """ Adjust polygon to be valid if not.

    Parameters
    -------
    poly : Polygon
        contour with shapely polygon format

    Returns
    -------
    valid_poly : Polygon
        contour with shapely polygon format

    """

    if poly.is_valid == True:
        valid_poly = poly
    else:
        valid_poly = poly.convex_hull

    return valid_poly



# -*- coding: utf-8 -*-

import numpy as np

__all__ = ["shift_cnt", ]

def shift_cnt(np_arr, shift_h=None, shift_w=None):
    """ Shift the position of contour.

    Parameters
    -------
    np_arr : np.array
        contour with standard numpy 2d array format
    shift_h : int or float
        shift distance in vertical direction
    shift_w : int or float
        shift distance in horizontal direction

    Returns
    -------
    shift_arr: np.array
        shifted contour

    """

    # construct new shift_arr from original array
    shift_arr = np.array(np_arr)
    # shift in vertical direction
    if shift_h != None:
        shift_arr[0] += shift_h

    # shift in horizental direction
    if shift_w != None:
        shift_arr[1] += shift_w

    return shift_arr



# -*- coding: utf-8 -*-

import numpy as np

from .coor_transform import swap_wh

__all__ = ["cv_cnt_to_np_arr",
           "np_arr_to_cv_cnt"]


def cv_cnt_to_np_arr(cv_cnt):
    """ Convert cv2 contour to numpy 2d array ([0]-h, [1]-w).

    Parameters
    -------
    cv_cnt : np.array
        contour with opencv format

    Returns
    -------
    cnt_arr : np.array
        contour with standard numpy 2d array format

    """

    cnt_arr = cv_cnt.squeeze().transpose()
    cnt_arr = swap_wh(cnt_arr)

    return cnt_arr


def np_arr_to_cv_cnt(np_arr):
    """ Convert numpy 2d array ([0]-h, [1]-w) to cv2 contour.

    Parameters
    -------
    cnt_arr : np.array
        contour with standard numpy 2d array format

    Returns
    -------
    cv_cnt : np.array
        contour with opencv format

    """

    np_arr = swap_wh(np_arr)
    arr_t = np_arr.transpose()
    cv_cnt = np.expand_dims(arr_t, axis=1)

    return cv_cnt



# -*- coding: utf-8 -*-

import sys
from os.path import dirname as opd
from os.path import abspath as opa
TEST_PATH = opa(opd(__file__))
PRJ_PATH = opd(TEST_PATH)
sys.path.insert(0, PRJ_PATH)

from pycontour.coor_transform import point_list_to_np_arr
from pycontour.coor_transform import np_arr_to_point_list


def test_coor_transform():
    point_list = [(0.0, 0.0), (2.0, 0.0), (2.0, 2.0), (1.0, 2.0)]
    np_arr = point_list_to_np_arr(point_list)
    new_points = np_arr_to_point_list(np_arr)
    if point_list != new_points:
        raise AssertionError("Conversion error")



# -*- coding: utf-8 -*-

import sys
import numpy as np
from os.path import dirname as opd
from os.path import abspath as opa
TEST_PATH = opa(opd(__file__))
PRJ_PATH = opd(TEST_PATH)
sys.path.insert(0, PRJ_PATH)

from pycontour.coor_transform import np_arr_to_point_list
from pycontour.poly_transform import np_arr_to_poly
from pycontour.poly_transform import point_list_to_poly
from pycontour.poly_transform import bbox_to_poly
from pycontour.poly_transform import poly_to_np_arr


def test_poly_transform():
    np_arr = np.array([[1., 2., 4., 5., 3.], [1., 3., 4., 2., 0.]])
    poly1 = np_arr_to_poly(np_arr)
    print("Bounds of poly1 is: ", poly1.exterior.bounds)
    point_list = np_arr_to_point_list(np_arr)
    poly2 = point_list_to_poly(point_list)
    print("Bounds of poly2 is: ", poly2.exterior.bounds)
    if poly1 != poly2:
        raise AssertionError("Conversion error")
    min_h, max_h = np.min(np_arr[0]), np.max(np_arr[0])
    min_w, max_w = np.min(np_arr[1]), np.max(np_arr[1])
    poly3 = bbox_to_poly(min_h, min_w, max_h, max_w)
    print("Bounds of poly3 is: ", poly3.exterior.bounds)
    # poly3.exterior.coords.xy
    poly1_arr = poly_to_np_arr(poly1)
    print("Numpy coordinates of poly1 is:")
    if not np.array_equal(np_arr, poly1_arr):
        raise AssertionError("Conversion error")



# -*- coding: utf-8 -*-

from ..poly_transform import np_arr_to_poly
from .intersect import construct_intersection_polygon
from ..poly import get_poly_area


def cnt_dice_ratio(cnt1, cnt2, smooth=0.01):
    """ Calculate the dice ratio between two contours.

    Parameters
    -------
    cnt1 : np.array
        contour with standard numpy 2d array format
    cnt2 : np.array
        contour with standard numpy 2d array format
    smooth: float
        value for smoothing dice ratio

    Returns
    -------
    dice_ratio : float
        dice ratio between two contours, ranging from 0.0 to 1.0

    """

    poly1 = np_arr_to_poly(cnt1)
    poly2 = np_arr_to_poly(cnt2)
    inter_poly = construct_intersection_polygon(cnt1, cnt2)

    poly1_area = get_poly_area(poly1)
    poly2_area = get_poly_area(poly2)
    inter_area = get_poly_area(inter_poly)

    dice_ratio = (2*inter_area + smooth) / (poly1_area + poly2_area + smooth)

    return dice_ratio



def cnt_jaccard_index(cnt1, cnt2, smooth=0.01):
    """ Calculate the jaccard index between two contours.

    Parameters
    -------
    cnt1 : np.array
        contour with standard numpy 2d array format
    cnt2 : np.array
        contour with standard numpy 2d array format
    smooth: float
        value for smoothing jaccard index

    Returns
    -------
    jaccard_index : float
        jaccard index between two contours, ranging from 0.0 to 1.0

    """

    poly1 = np_arr_to_poly(cnt1)
    poly2 = np_arr_to_poly(cnt2)
    inter_poly = construct_intersection_polygon(cnt1, cnt2)

    poly1_area = get_poly_area(poly1)
    poly2_area = get_poly_area(poly2)
    inter_area = get_poly_area(inter_poly)

    jaccard_index = (inter_area + smooth) / (poly1_area + poly2_area - inter_area + smooth)

    return jaccard_index



# -*- coding: utf-8 -*-

import os, sys
import numpy as np
from scipy.ndimage import binary_fill_holes
from skimage import io
from skimage import img_as_ubyte
from skimage.morphology import remove_small_objects
from skimage.morphology import disk, binary_closing
import cv2

from os.path import dirname as opd
from os.path import abspath as opa
TEST_PATH = opa(opd(__file__))
PRJ_PATH = opd(TEST_PATH)
sys.path.insert(0, PRJ_PATH)
from pycontour.cv2_transform import cv_cnt_to_np_arr, np_arr_to_cv_cnt


def extract_cnt_using_cv2(img_path):
    img = io.imread(img_path)
    gray = np.dot(img, [0.299, 0.587, 0.114])
    binary = gray < 200
    no_small = remove_small_objects(binary, min_size=5000, connectivity=8)
    fill = binary_fill_holes(no_small)
    selem = disk(10)
    mask = binary_closing(fill, selem)
    ubyte_mask = img_as_ubyte(mask)
    cnts, _ = cv2.findContours(ubyte_mask, mode=cv2.RETR_EXTERNAL, method=cv2.CHAIN_APPROX_NONE)

    return cnts


def test_cv_np_transfom():
    img_path = os.path.join(TEST_PATH, "data/Imgs/20181218042607.jpg")
    cnts = extract_cnt_using_cv2(img_path)
    test_cnt = cnts[0]
    # convert cv2 contour to numpy array
    np_arr = cv_cnt_to_np_arr(test_cnt)
    # convert numpy arrary to cv2 contour
    cv_cnt = np_arr_to_cv_cnt(np_arr)

    if not np.array_equal(test_cnt, cv_cnt):
        raise AssertionError("Conversion back and forth error")



# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

import pycontour

PKG_NAME = "pycontour"
VERSION = pycontour.__version__
DESCRIPTION = "pycontour: A Python Toolkit for 2D Contour Processing"
HOMEPAGE = "https://github.com/PingjunChen/pycontour"
LICENSE = "BSD 3-Clause"
AUTHOR_NAME = "Pingjun Chen"
AUTHOR_EMAIL = "pingjunchen@ieee.org"

REQS = ""
with open('requirements.txt') as f:
    REQS = [pkg for pkg in f.read().splitlines()]

CLASSIFIERS = [
    'Development Status :: 3 - Alpha',
    'Intended Audience :: Developers',
    'Intended Audience :: Healthcare Industry',    
    'Intended Audience :: Information Technology',
    'Intended Audience :: Science/Research',
    'License :: OSI Approved :: BSD License',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
    'Topic :: Scientific/Engineering',
    'Topic :: Scientific/Engineering :: Mathematics',
    'Topic :: Scientific/Engineering :: Artificial Intelligence',
]

args = dict(
    name=PKG_NAME,
    version=VERSION,
    description=DESCRIPTION,
    url=HOMEPAGE,
    license=LICENSE,
    author=AUTHOR_NAME,
    author_email=AUTHOR_EMAIL,
    packages=find_packages(),
    install_requires=REQS,
    classifiers= CLASSIFIERS,
)

setup(**args)
